"""_inmidas.py: main PyMidas MIDAS environment initialization
 
Sets MIDAS global system environment variables.


"""
##############################################################################
#
#   This file is part of PyMidas.
#
#   PyMidas is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   PyMidas is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMidas; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
##############################################################################
#
#  Author:             S. Maisala       
#  Email:              pymidas@eso.org
#  Contact:            www.eso.org/sampo/pymidas
#
##############################################################################
#
#   Change History log
#
# Num  Date      Revision  Who   What
#
# 00   2005/07/15  0.95    SM   Creation of this History  
# 
# 01   2005/11/03  1.0.0   SM  Version 1.0.0
#
###############################################################################


def environment( _debug ):
    import os
    # Setting all other midas envinroment variables
    MIDASHOME=os.environ['MIDASHOME0']
    MID_HOME=os.environ['MIDASHOME']+'/'+os.environ['MIDVERS']

    os.environ['graph_term']='tt'
    os.environ['MID_HOME']      = MID_HOME

    PIPE_HOME=MID_HOME+'/pipeline'
 
    os.environ['PIPE_HOME']     = PIPE_HOME
    os.environ['MID_INCLUDE']   = MID_HOME+'/incl'
    #
    os.environ['SYS_EXE']       = MID_HOME+'/system/exec'
    os.environ['MID_EXE']       = MID_HOME+'/prim/exec'
    os.environ['APP_EXE']       = MID_HOME+'/applic/exec'
    os.environ['STD_EXE']       = MID_HOME+'/stdred/exec'
    os.environ['CON_EXE']       = MID_HOME+'/contrib/exec'
    os.environ['GUI_EXE']       = MID_HOME+'/gui/exec'
    os.environ['UVES_EXE']      = PIPE_HOME+'/uves/exec'
    #
    MID_PROC=MID_HOME+'/prim/proc'
    os.environ['MID_PROC']     = MID_PROC     # executed via @
    os.environ['APP_PROC']     = MID_HOME+'/applic/proc'  # executed via @a
    os.environ['STD_PROC']     = MID_HOME+'/stdred/proc'  # executed via @s
    os.environ['CON_PROC']     = MID_HOME+'/contrib/proc'  # executed via @c
    os.environ['GUI_PROC']     = MID_HOME+'/gui/proc'     # executed via @g
    os.environ['XDO_HELP']     = MID_HOME+'/gui/XDo/help'
    #
    os.environ['MID_MONIT']    = MID_HOME+'/monit'
    os.environ['MID_CONTEXT']  = MID_HOME+'/context'
    os.environ['MID_SETUP']    = MID_PROC+'/setup'
    os.environ['MID_LIB']      = MID_HOME+'/lib'
    os.environ['XAPPLRESDIR']  = MID_HOME+'/gui/resource'
    #
    os.environ['MID_SYSTAB']   = MID_HOME+'/systab/bin'
    os.environ['AGL3CONFIG']   = MID_HOME+'/systab/ascii/plot/'
    os.environ['MID_HELP']     = MID_HOME+'/prim/help'
    os.environ['MID_FIT']      = MID_HOME+'/applic/fit/src'
    #
    os.environ['MID_CASPEC']   = MIDASHOME+'/calib/data/inst/caspec'
    os.environ['MID_EFOSC']    = MIDASHOME+'/calib/data/inst/efosc'
    os.environ['MID_IRSPEC']   = MIDASHOME+'/calib/data/inst/irspec'
    os.environ['MID_ARC']      = MIDASHOME+'/calib/data/spec/line'
    os.environ['MID_STANDARD'] = MIDASHOME+'/calib/data/spec/flux'
    os.environ['MID_EXTINCTION']= MIDASHOME+'/calib/data/spec/exti'
    os.environ['MID_PEPSYS']   = MIDASHOME+'/calib/data/pepsys'
    os.environ['MID_FILTERS']  = MIDASHOME+'/calib/data/datatrans'
    os.environ['MID_EMMI']     = MIDASHOME+'/calib/data/inst/emmi'
    os.environ['MID_FEROS']    = MIDASHOME+'/calib/data/inst/feros'
    #
    os.environ['MID_TEDIT'] = os.environ['MID_HELP']+'/tbledit.twh'
    #
    # define logical symbols for demo areas
    #
    os.environ['MID_TEST'] = MIDASHOME+'/demo/data'
    os.environ['RAW_DEMO'] = MIDASHOME+'/demo/raw'
    #
    # From start.csh:
    os.environ['TERMCAPFILE'] = os.environ['MID_INCLUDE']+'/termcapfile'
    os.environ['DEVCAPFILE']  = os.environ['MID_INCLUDE']+'/devcap.dat'
    os.environ['PARALLEL']    =''
    os.environ['PYMIDAS'] = 'YES'
    # if necessary create MIDAS work directory

    if os.environ.has_key( 'MID_WORK' ):
        if _debug:
            print 'MIDAS working directory', os.environ[ 'MID_WORK' ]
        path = os.environ[ 'MID_WORK' ]
        if path[-1] <> '/':
            path=path+'/'
    else:
        if os.path.exists( os.path.join( os.environ[ 'HOME' ], 'midwork' ) ):
            path = os.path.join( os.environ[ 'HOME' ], 'midwork' )
            if _debug:
                print 'MIDAS default working directory:',path
            path  = path+'/'
        else:
            if _debug:
                print 'MIDAS working directory does not exist!'
            path=os.path.join( os.environ[ 'HOME' ],'midwork/' )
            os.mkdir(path)
            if _debug:
                print 'Creating default working directory:', path

    os.environ[ 'MID_WORK' ]  = path

