""" PyMidas commands module. Here we have all pymidas commands

S. Maisala  2005 June 15

"""

##############################################################################
#   This file is part of PyMidas.
#
#   PyMidas is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   PyMidas is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with PyMidas; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
##############################################################################
#
#  Author:             S. Maisala       
#  Email:              pymidas@eso.org
#  Contact:            www.eso.org/sampo/pymidas
#
##############################################################################
#
#   Change History log
#
# Num  Date      Revision  Who   What
#
# 00   2005/09/19  0.95    SM   Creation of this History  
#
#
# 01   2005/09/19  0.95    SM   SOMO-6GDCB6
#                               Getting back MIDAS output into scripts.
#                               
#                               The last element in parse_result is missing.
#                               the element were lost by index error.
#
# 02   2005/09/22  0.95    SM   SOMO-6GFB4Z
#                               Keyword routines problems - and crash
#                               Buffer overflow fixed in C- layer and
#                               _putKeyVal  code improved to cut decimals
#                               in same lenght as in command string
#
# 03  2005/09/22   0.95    SM   create/command in login.prg code improved
#                               to accept format create/command kplot @@ myplot
#
# 04 2005/09/28    0.95    SM   SOMO-6GNBV9
#                               tutorial... command was not working
#
# 05 2005/11/03    1.0.0   SM   New release with some extra PyMidas routines 
#
# 05 2006/02/28    1.0.5   SM   New release with external (shell)commands
#                               try .. except bug in subprocess failure fixed.

###############################################################################


import sys, os, types, _pymidas_c_00, _pymidasInit
import midas, midastask, midas, subproc, time

_pymidas=_pymidas_c_00
_subproc = subproc
_os = os

# Hide modules
del subproc 
del _pymidas_c_00


try:
    class PyMidasError( Exception ):
        pass
 
except TypeError:
    # string based exceptions
    PyMidasError = 'PyMidasError'




class globalVars:
    pass
    variable = 0
    imported = 0
    mimic = 0
    debug = 0
    socket = 1
    running = 0
    commands =[]
    programs = []    
globals = globalVars()


sys.path.append(os.environ['MID_WORK'][:-1])




def _do( command ):
    # send command to background  midas    
    
    # set background midas timeout level
    _pymidas.SetParam ('MIDAS_CMD_TIMEOUT','0')
    if globals.debug:
        print "command sent to Midas: ", command 
    # send command to bkg midas
    status = 0 
    
    midStatus,result = _pymidas.Do( command  )
    i = 0
    status = -1
    
    while status!= 0:
        time.sleep( 0.1 )
        
        if midStatus == 56:   # value 56 comes from midas if parameter is needed
            if globals.debug:
                print 'Midas needs a a parameter...'
            paramValue=_getParam(result)
            if paramValue :
                if paramValue =='empty': return
                else:
                    newCommand= command+' '+ paramValue                
                    result = _do( newCommand )
                    return result
        status,midStatus,result = _pymidas.GetMidasStatus()

        if status == 99:  # New commands 
             if globals.debug:
                print 'Midas commands database is changed!'
             readCommands()
             initTasks()
             status=0
             return result


        if status == 98:  # Commands are deleted 
            if globals.debug:
                print 'Midas command is deleted'
                print command
            deleteCommand(command)
            status=0
            return result


        
        
        i =i+1
        if midStatus < 0:
            print 'MIDAS unknown error:', midStatus
            return ''


        if i == 500:
            print 'Command tried to send 500 times to background Midas...'

        if i == 1000:
            print 'Command tried to send 1000 times to background Midas...'
            print 'Problem... Check if background Midas is alive!'
            return
    if midStatus == 0:
        askParam =0

        return result
        #return parse_result(result)
    
    elif midStatus > 0:
        _promptError(midStatus)
        return ''
    
    


        
def _run( string ):
    """ Run Midas command"""
    status =0
    result =''

    if string =='':    
        return ''
    if string == 'bye':
        if globals.imported:
            _pymidas.Do( 'CLEAR/BACKGROUND 2; BYE' )
            _pymidasInit.cleanSession(1)
            sys.exit()
            return
        else:
            print 'Use .exit to exit'
            return '' 
    if string[0] =='$':
        string[0]=='!'

    string = parser( string )
    command,index = validateCommand( string )
    
    
    # index tells the program file place in programlist
    if index > 0:

        if globals.socket:
            if globals.debug:
                print "command sent to Midas: ", string
            result = _do( string )

        else:    
            if globals.debug:
                print "Warning! Parameter asking is not fully supported in files mode!" 
            parameters = validateParameters( command ,index[0])
            sendcomm = ''

            # put all in sendcomm string
            for pars in parameters:
                sendcomm = sendcomm+ pars + ' '                 
    
            result = _do( sendcomm )

        # Check if output is directed into variable 
        if globals.variable:
            return parse_result(result)
        else: return 
    else:
        if string !='':
            result = _do( string )
            
            #return -1,  string +' is not valid  Midas command'
            if not globals.mimic:
                print string +' is not valid  PyMidas command'
            return ''


def parser( string ):
    """ Parsing Python style code in Midas syntax"""
 
    # Lower all
 
    #string = string.lower( )
    string = string.rstrip( ) # remove spaces from right
    # check, if Python style or plain Midas command
    if string[ 0:6 ] != "midas.":
        ik = string.find( " " )
        if ik > 1:   # We have a parameter
            if  globals.mimic:
                command = string[ :ik]
 
            else:
                # Change command in python syntax
                command = py2midas( string[ :ik ])
                
            parameters = string[ ik: ]
            return command + parameters
 
        else:      # we have only command
            
            
           if  globals.mimic:
               command = string.upper()
           else:
               # Change command in python syntax
               command = py2midas( string )
        return command

    else:return command


def parse_result(result):
    """ Parse midas produced output in python style"""
    if result =='':
        return ''
    
    if result =='   \n':
        return ''
    # First remove new line and '\t' marks    
    result = result.replace('\n',' ')
    result = result.replace('\t','')
    result  = result.replace('(','')
    result  = result.replace(')','')
    result  = result.replace(',',' ')
    
    result = result.split(' ') 
    # Remove empty list objects
    elements =[]
    for i in result: 
        if i != '':
            elements.append(i)

    total =[]
    result =[]
    i =0
    if len(elements) < 1:
        return ''
    #print elements, len(elements)

    # special case read/keyword 
    if elements[i] =='keyword:':
        i = i+2
        while  i <  len(elements):  # bug (01) len(elements) -1
        
            if elements[i] == 'type:':
                i=i+1 
                type = elements[i]
                i = i + 1
                i = i + 1
                num_elements =elements[i]    
                i = i+1

            
                    
            if 'keyword:' not in elements[i] and i <  len(elements):
    

                if 'float' in type:
                    result.append( float ( elements[ i ] ) )
                
                elif 'double' in type:
                    result.append( float ( elements[ i ] ) )
                    
                elif 'integer' in type:
                    result.append( int ( elements[ i ] ) )

                elif 'real' in type:
                    result.append( float ( elements[ i ] ) )

                else:  result.append( elements[ i ]  )
                
                i =i+1
            else:
            
                i = i + 2

    else:
        while  i <  len(elements):
            result.append(elements[i])                   
            i =i +1
                

    return result




def validateCommand(string):
    """ Validate command is really MIDAS command
    If we found the Midas command we will return
    the command string and its place in command list  
    """
    
    string = string.rstrip()
    if ' ' in string:
        Token = string.split(' ')
        command = Token[0]
        #command = command.replace( "_", "/" ) # Replace _ with slash
        command = py2midas( command )
    else:
        command = string
        command = py2midas( string )
        #command = command.replace( "_", "/" )         # Replace _ with slash
    return MidasCompleter(command)


def validateParameters(string, index):
    """   Validate the need of parameters. If midas command should have a
    parameter(s) we will read them in.
    """
    if ' ' in string:
        Token = string.split(' ')
        numParams = len(Token)
        command = Token[0].upper()

    else:
        Token = string.upper()
        command = string.upper()
        numParams = 0
        
    #print(numParams)    
    Params = MidasPrograms()    
    program = Params[index]
    if program =='None':  # We do not have a program file
        parameters=[]

        if numParams > 0:
            for string in Token:
                parameters.append( string ) # We just return command!!
                parameters[ 0 ] = command
        else: parameters.append( command )        
        return parameters

    else:
        filename = findProgFile( program )
        if globals.debug:
            print command ,program,numParams
        parameters = getMidParams( command,program, numParams )

        i =0
        if numParams > 0:
            for string in Token:
                parameters[ i ] = string
                i = i+1
            parameters[ 0 ] = command

        return parameters
        
    




def MidasCompleter(text):
    """    Complete midas commands
    If input text matches of any midas command(s) this function
    returns possible midas commands
    """
    result,location =  primary_matches(text)
    if len(result) == 1: # we have only one match
        return text, location
    else:
        # Check if one of the command length is equal to results 
        for i in result:
            if text == i:
                return primary_matches(text)
        # In this case we don't have a match 
        return result, -1
    




def primary_matches(text):
    """   Return matches when text is at beginning of the line

    It will also return the list index number (location) 
    """
    
    matches = []
    location = []
    n = len(text)
    commands = MidasCommands()
    place = 0
    for word in commands:  
        if word[:n].lower() == text:
            matches.append(word.lower())
            location.append(place)
        place = place +1    
    return matches,location



def midas_matches(text):
    """   Return matches when text is at beginning of the line 
    """
    
    matches = []
    location = []
    n = len(text)
    commands = MidasCommands()
     # Add Extra commands in database
    commands.append( 'cd' )
    commands.append( 'pwd' )
    commands.append( '@' )
    commands.append( '@@' )
    #commands.append( 'get/pix')
    place = 0
    for word in commands:
        
        if globals.mimic:
            text=text.upper()
        else:
            word=word.lower()
            word = midas2py(word)
        #word = word.replace('/','') # replace / to python style            
        if word[:n]  == text:
            matches.append(word)
    return matches


def midas_task_matches(text):
    """   Return matches when text is at beginning of the line 
    """
    
    matches = []
    location = []
    n = len(text)
    commands = MidasCommands()
    # Add Extra commands in database
    #commands.append( 'get/keyword' )
    commands.append( 'put/keyword' )
    commands.append( 'get/pix')
    commands.append( 'put/pix')
    commands.append( 'get/desc')
    commands.append( 'put/desc')

    
    commands.append( 'do' )
    commands.append( 'cd' )
    commands.append( 'pwd' )
    
    place = 0
    for word in commands:
        word=word.lower()
        word = midas2py(word)
        if word[:n]  == text:
            matches.append(word)            
    return matches


def midas2py(command):
    """ Change midas style command in python style"""
    i = command.find('/')
    if i >0:
        comPre = command[0:i]
        comCap = command[i+1].upper()
        comPost = command[i+2:]
        command = comPre + comCap + comPost

    return command

def py2midas(command):
    """ Change Python style command in midas style"""
        
    i = -1
    for x in range(len( command )):
        if command[x].isupper():
            i = x 
            if i >0:
                comPre = command[0:i]
                comLow = command[i].lower()
                comPost = command[i+1:]
                command = comPre + '/' + comLow + comPost
    return command



def _getParam( parameterStr ):
    if '"' in parameterStr:
        paramtmp = parameterStr.split('"')
        paramNum= paramtmp[0]
        paramQuest =paramtmp[1]
    
        if parameterStr =='':
            return 'empty'
    elif parameterStr =='?':   
        paramQuest = ':'
    string = ''
    try:       
        string = str( raw_input( paramQuest + ' ' ))
    
    except ValueError:
        print 'Oops! Not a valid string'
    if string =='':
        print '\x1B[0;31m' + 'DEFINE/PARAM '+parameterStr
        print'             ^^'+ '\x1B[0m'
        _promptError(31)
        return 'empty'
    return string
    
    


# Print Error messages out    

def _promptError(status):
    """ If bkg midas status not 0 --> prompt error"""
    # We have novice or Expert errors
    # default is Novice

    #print 'status:',status,'#' 
    if status > 0 and status <= 110 : # We have 110 errors 
        nov,exp,novnum,expnum = MidasErrors()
        err= novnum.index('NOVICE.'+ _modifyNum(status))
        #print err
        print '\x1B[0;31m'+ nov[err+1] + '\x1B[0m'
    else:
        print ''
        #print 'Pymidas C-layer error:',status


# Modify numbers as 3-digit format for Errors

def _modifyNum(num):
    """ Create 000 format number"""
    if len( str( num ) ) == 1:
        string='00'+str(num)
    if len( str( num ) ) == 2:
        string='0'+str(num)
    if len( str ( num ) ) == 3:
        string = str( num )
    if  len( str ( num ) ) > 3 or   len( str ( num ) ) == 0 :
        string ='000'
    return string



# Returns all possible combinations of Midas commands

def MidasCommands():
    """ This function returns all Midas commands"""
    comms,progs = globals.commands, globals.programs
    type(comms)
    modcomms=[]
    # Cut the latter part of command length to four letters
    for word in comms:
        
        pref = word.split('/')
        if len( pref ) > 1:            
            post = word.split('/')[1]
            if len( post ) > 4:
                post = post[:4]
            word = pref[0] + '/' + post 
        
        modcomms.append( word )
    
    return modcomms

# Delete commands


def deleteCommand(command):
    comm = command.split(' ')
    #print comm[0]
    if comm[0:11] == 'delete/comm':

        if len(comm) > 1:
            delCommand = command.split(' ')[1]
            delCommand = delCommand.upper()
    
            if delCommand in globals.commands:
                # print delCommand
                globals.commands.remove( delCommand )
                initTasks()        
    else:
         readCommands()
         initTasks()



# Returns Midas programs


def MidasPrograms():
    """ This Function returns all Midas *.prg files"""
    ccomms,progs = globals.commands, globals.programs
    return progs


# Here we read midas commands file MID_MONIT/newcom.in,
# which has list of all midas commands and program files

def readCommands():
    """ Read all Midas commands from file MID_MONIT/newcom.in.
        Parse out all commands and white spaces"""
    
    filename = _os.path.join( os.environ[ 'MID_MONIT' ],'newcom.in' )
    if _os.path.exists(filename):
        file = open( filename, 'r' )
        line = 'notEmpty'
        commands = []
        numCommands = 0
        while line != '':
            line = file.readline( )
            string = line
            if '!' in string: # Take out comments
                cut = string.index( '!' )
                string = string[ 0: cut ]

            string.replace( '\t', '' ) # Take out tabs

            string.replace( '\n', '' ) # Take out EOL

            
            result = string.rstrip()
            # (remove original TUTORIAL command 
            if result =="TUTORIAL/... @ tutorial":
                result =""

            if result =="PLAYBACK/...":
                result =""
            if result =="CONTINUE/...":
                result =""


                
            if result != "":
                commands.append( result )
        file.close( )
        # Add Tutorial commands (4) SOMO-6GNBV9
        
        commands.append( "TUTORIAL/AVERAGE  @ tutorial" )
        commands.append( "TUTORIAL/EXTRACT @ tutorial " )
        commands.append( "TUTORIAL/FILTER  @ tutorial" )
        commands.append( "TUTORIAL/GRAPHICS  @ tutorial" )
        commands.append( "TUTORIAL/HELP @ tutorial" )
        commands.append( "TUTORIAL/ITT @ tutorial" )
        commands.append( "TUTORIAL/LUT @ tutorial" )  
        commands.append( "TUTORIAL/SPLIT@ tutorial " )
        commands.append( "TUTORIAL/TABLE @ tutorial" )
        commands.append( "TUTORIAL/ALIGN @ tutorial " )
        commands.append( "TUTORIAL/FIT @ tutorial" )   
        commands.append( "CONTINUE/CLEAR" )
        commands.append( "PLAYBACK/FILE" )
        commands.append( "PLAYBACK/LOGFILE" )

       

        

        if globals.running:
            _pymidas.SetParam ('COMM_OUT','0')
            usercommands = _do('SHOW/COMM')
            _pymidas.SetParam ('COMM_OUT','1')

            tempstring = usercommands.split('\n')
            for string in tempstring:
                if '!' in string: # Take out comments
                    cut = string.index( '!' )
                    string = string[ 0: cut ]
                    

                string.replace( '\t', '' ) # Take out tabs

                #remove spaces
                #Finally strip out white spaces

                result = string.rstrip()
                
                if result != "":
                    commands.append( result )
                    
                
        # Parse out commands and programs
        progList = []
        commList = []
        progParams=[]
        for string in commands:
            if '@' in string: # we MUST have a program file
                cut = string.index( '@' )
                command = string[ 0 : cut ]
                
                command = command.rstrip( ) # strip out white spaces
                if ('(') in command:   # remove ( number)
                    command = command.split('(')[0]
                #(03) accept format create/command kplot @@ myplot
                if '/' in command:         # check if blanks before /
                     cut = command.index('/')
                     pre = command[ 0: cut ]
                     if ' ' in pre:         #  cut pre command after space
                         cut1 = pre.index(' ')
                         pre = pre.rstrip()
                     post = command[cut+1 :]
                     post = post.rstrip() # take out blanks
                
                     if post =='':
                         command = pre[0:]
                     else:
                         command = pre +'/' + post
                #print command
		if ' ' in command:         #  cut command after space
                    cut = command.index(' ')
                    command = command[ 0 : cut]
                #print command
                     
                commList.append( command )  # put command in list
                
                # next parse programs
                program = string[ cut :  ]
                #print program
                # cut marks before program
                if ' ' in program:         #  cut command after space
                    cut = program.index(' ')
                    program = program[ cut+1: ]
                    #print program
                if ' ' in program:         #  cut command after space
                    cut = program.index(' ')
                    program = program[ 0 : cut]
                    #print program
                progList.append( program ) # put program name in list
                
            else:
                if ' ' in string:         #  cut command after space
                    cut = string.index(' ')
                    string = string[ 0 : cut] 
                commList.append( string ) # put command in list
                progList.append( 'None' ) # put program name in list
                progParams.append( 0 )
                

        globals.commands = commList
        globals.programs = progList
        return commList, progList

    else:
        raise SyntaxError( "Midas commands database not found" )



def initTasks():
        comms = MidasCommands()
        for tasks in comms:
            tasks = tasks.lower( )
            tasks = midas2py(tasks)
            setattr( midas, tasks, midastask.MidasTask(tasks) )
        # Add Pymidas Extra commands
        setattr( midas, 'do', _do ) 
        #setattr( midas, 'getKeyword' , pymidasCom._getKeyVal )
        setattr( midas, 'putKeyword' , _putKeyVal )
        setattr( midas, 'getPix' , _getPix )
        setattr( midas, 'putPix' , _putPix )
        setattr( midas, 'getDesc' , _getDesc )
        setattr( midas, 'putDesc' , _putDesc )



# This function reads Midas errors from syserr.dat file
# File has two kind of levels of errors. The error numbering is
# same, but NOVICE errors  gives more specific information of
# error than EXPERT.

# Note: WE use NOVICE as default.--> possible to add flag
# novice/expert to choose of error level
# This syntax is cumbersome. There are missing error numbers in chronological,
# order, so we have to carry number information with us


def MidasErrors():
    """   Read Midas errors from syserr.dat file
    File has two kind of levels of errors. The error numbering is
    same, but NOVICE errors  gives more specific information of
    error than EXPERT.

    returns:
    NovErrors: list of Novice errors
    ExpErrors: list of Eexpert errors
    NovNumber: list of Novice error numbers
    ExpNumber: list of Expert error numbers
    
    """

    filename = _os.path.join( _os.environ[ 'MID_MONIT' ],'syserr.dat' )
    if _os.path.exists(filename):
        file = open( filename, 'r' )
        line = 'notEmpty'
        NovErrors = []
        ExpErrors = []
        NovNumber = []
        ExpNumber = []
        error=''
        while line != '':
            line = file.readline( )
            string = line
            # First Take EXPERT errors
            
            if 'EXPERT' in string: # We have a new error
                line = line.replace( '\n','') # Take out newline
                ExpNumber.append( line )
                ExpErrors.append(error)
                error =''
            # Then we take NOVICE errors    
            elif 'NOVICE' in string: # We have a new error
                line = line.replace( '\n','') # Take out newline
                NovNumber.append( line )
                NovErrors.append(error)
                error =''
                
            else: 
                error = error + line            
              
        file.close( )
    return NovErrors, ExpErrors, NovNumber, ExpNumber






# This function finds the actual Midas program file from MIDAS
# system. If succesfull the filename with full path will be returned.

def findProgFile(program):
    """   Find the actual Midas program file from MIDAS installation path.
    The input parameter is taken out from /MONIT/newcom.in file
    If succesfull the filename with full path will be returned.

    input parameter: progname
    output: filename
    on error: empy string 
       
       
    """

    name = program+'.prg'
    # Find file from midas paths
    filename = _os.path.join( _os.environ[ 'MID_PROC' ],name )
    if _os.path.exists( filename ):
        return( filename )
        
    
    filename = _os.path.join( _os.environ[ 'APP_PROC' ],name )
    if _os.path.exists( filename ):
        return( filename )

    filename = _os.path.join( _os.environ[ 'STD_PROC' ],name )
    if _os.path.exists( filename ):
        return( filename )

    filename = _os.path.join( _os.environ[ 'CON_PROC' ],name )
    if _os.path.exists( filename ):
        return( filename ) 

    filename = _os.path.join( _os.environ[ 'GUI_PROC' ],name )
    if _os.path.exists( filename ):
        return( filename )
    
    print "Midas program", name, " not found"
    return ''







    
def getMidParams( command, program, numParams ):
    """ Read in Midas Command parameters. Open corresponding MIDAS program file and
    covert parameter querying in python code. Import and execute midas code and return
    command + parameters. If numParams > 1 we do not read parameter 1 etc.."""
    outfile = _os.path.join( _os.environ[ 'MID_WORK' ],'midPar.py' )
    
    filename = findProgFile(program)
    #print filename
    if '/' in command:
        action = command.split('/')[1]
    else:
        action = command
    match = ['if action(','elseif action('] # make if logic if possibe 
    parameters = 16 * [''] # Midas definition max 8 params
    line = 'not Empty'
    if _os.path.exists( filename ):
        file = open( filename, 'r' )

        fileout =   open( outfile, 'w' )
        flag =1
        line1=''
        fileout.write('def readMidPar( action ):\n')
        fileout.write('     parameter = 16 * [""]\n')
        #fileout.write('     print action\n')
        fileout.write('     parameter[0] = "' + command + '"\n')
        while line != '':
            if flag:
                line = file.readline( )
                #print line
            if flag == 0:
                flag = 1
                line=line1
                #print line
            output=''
            for test in match:
                 
                if test in line:
                    output=line.replace( '!', '#' )
                    output = output.replace( '(', '[' )
                    output = output.replace( ')', ']' )
                    output = output.replace( '.eq.', '==' )
                    output = output.replace( 'elseif' , 'elif')
                    output = output.replace( 'then' , ':')
                    

                    # replace action[x:x] to python compactible format 
                    if '[' in output:
                        minp = output.index('[')+1
                        minvalue = int( output[minp] ) -1
                        oldstr= output[minp]
                        newstr= str( minvalue )                     
                        output = output.replace( oldstr,newstr,1)          

                    
                    line1 = file.readline() # read next line if define all ok
                    # print line1
            
                    
                    if 'define/param p' in line1:
                        if '?' in line1 or '{p' in line1:                        
                            fileout.write('     '+output+'\n')
                            #print '#     '+output+'\n'
                            while  'define/param p' in line1:                        
                                output1= defpar2py( line1 , numParams)
                                print '#     '+output1
                                fileout.write('     '+output1+'\n')
                                line1 =file.readline()
                                print line
                                flag = 0


                else:
                    if 'define/param p' in line:
                        #print line
                        while  'define/param p' in line:
                            line=line.lstrip()
                            output1 = defpar2py( line , numParams )
                            #print '#     '+output1
                            fileout.write('     '+output1+'\n')
                            line =file.readline()
                            #print line
                            line1=line
                            flag = 0
                            
    fileout.write('     return parameter\n')
    file.close()
    fileout.close()

    
    
    import midPar  # import produced python file
    reload( midPar ) # reload to get newest file
    parameters = midPar.readMidPar(action) # put values in parameters
    # remove temporary python file
    if os.path.exists(outfile):
        os.remove( outfile )
    return parameters


def defpar2py( line, numParams ):
                     
     output='' 
     #print '#parameter in',line
     templine = line.split( 'define/param p')[1]
     if templine[2] in '?' or templine[2] in '{p'  : # Must parameter
         
         if '?' in line:
             mark=line.index('?')
    
         elif '{p' in line:
             mark=line.index('{p')

         else:
             mark=2    
         parNum = int ( line.split( 'define/param p' )[ 1 ][ 0 ] )
         if '+' in line[ mark - 2 ] or 'N' in line[ mark - 2 ] :
             # if + sign no MUST parameter
             #print 'no MUST input parameter'
             output = ''
                                     
         else:
             parNum = ( line.split( 'define/param p' )[ 1 ][ 0 ] )
             #print 'parNum = ',int(parNum)
             if int( parNum ) > numParams: 
                 output=line.replace( 'define/param p','*' )
                 output = output.split('*')[0]
                 paramquest =''
                 tempstr=line.split('"')
                 paramquest = tempstr[ 1 ]
                 paramquest.rstrip()  # Take spaces off
              
                 out1 = output + 'try:\n'
                 out2 = output + '         string = str( raw_input("' + paramquest + '"))\n'  
                 out3 = output + '     except ValueError:\n'
                 out4 = output + '         print "Not a valid string"\n'
                 out5 = output + '     parameter[' + parNum +'] = string\n' 
                 paramquest =''
             
                     
                 output=out1+out2+out3+out4+out5
             else:

                 output=line.replace( 'define/param p','*' )
                 output = output.split('*')[0]                 
                 #output = output +'print "parameter num must be >'+str(numParams)+ '"'
     #print '#parameters:',output
     return  output
                             
def ReadParametr(paramQuest):
    """Read asked parameter value"""
    if paramQuest =='':
        return ''
    
    string = ''
    while string =='':
        try:       
            string = str( raw_input( paramQuest  ))
            
        except ValueError:
            print 'Oops! Not a valid string'
    return string





def _getPix(image,xp,yp):
     midascommand = 'READ/IMAG ' +image+' @'+str(xp)+',@'+str(yp)+',1'     
     result = _do ( midascommand )
     result = result.strip()
     #print '*'+result+'*'
     if result != '':
         return float ( parse_result( result )[14] ) 
     else: return




def _putPix(image,xp,yp,value):
     midascommand = 'WRITE/IMAG '+image+' @'+str(xp)+',@'+str(yp)+',1 '+str(value)+' ALL'
     #print midascommand
     result = _do ( midascommand )
     midascommand = "LOAD/IMAG " +image 
     result = _do ( midascommand )
     return



def _getDesc(image,descr):
    """Read descriptor values from given image"""
    midascommand = 'READ/DESC ' + str(image) + ' '+ str(descr)   
    result= _do( midascommand)
    output= ( parse_result( result )[9:])
    if len(output) ==1:
        return output[0]
    else:
        return output

def _putDesc(image, descr, *values):
    """ Create or modify descriptor values in given image""" 
        

    if type( descr ) != types.StringType:
        raise SyntaxError( 'Keyword not string' )

  
    if len(descr) == 0 or len(values) == 0:
        raise PyMidasError("PutDescr requires  keywordname,value(s) ")


    keywordType = _validateValue( values) # Check values
    numValues = len ( values )

    # We do not allow multiple String words         
    if keywordType == types.StringType:
        if numValues > 1:
            raise SyntaxError( 'Cannot give multiple strings !' )
        midastype='C'


    midasValues=[]
    for i in values:
        newvalue = str(i)
        midasValues.append(newvalue)
        
       
    # Float or Double type
        
    if keywordType == types.FloatType:
            
        if len( str( midasValues[ 0 ] ) )  > 7:
            midastype = 'D'
        else:
            midastype = 'R' 

    # Int type

                       
    if keywordType == types.IntType:
        midastype='I'


    if  midastype == 'C':
        numValues= len( str( midasValues) )

        
    print 'Descr: ', descr     
    print 'no elements: ',numValues
    print 'type: ',midastype
    print 'values: ',midasValues

    #  (02) improved to cut decimals in same lenght as command string
    outputValues = ''   
    for i in midasValues:
        outputValues = outputValues + ',' + i   


    outputValues = outputValues[1:] # remove firts ','
        
    midascommand= "WRITE/DESC " + image + ' ' + descr+'/' + midastype+'/1/' + \
                  str( numValues) + ' ' + outputValues 
    _do ( midascommand )
        
    return

    
  

def _putKeyVal( string, *values):
    """ Put values in keyword database"""
    try:
        

        if type( string ) != types.StringType:
            raise SyntaxError( 'Keyword not string' )

  
        if len(string) == 0 or len(values) == 0:
            raise PyMidasError("PutKeyVal requires  keywordname,value(s) ")


        keywordType = _validateValue( values) # Check values
        numValues = len ( values )

        # We do not allow multiple String words         
        if keywordType == types.StringType:
            if numValues > 1:
                raise SyntaxError( 'Cannot give multiple words!' )
            midastype='C'


        midasValues=[]
        for i in values:
            newvalue = str(i)
            midasValues.append(newvalue)
        
       
        # Float or Double type
        
        if keywordType == types.FloatType:
            
                if len( str( midasValues[ 0 ] ) )  > 7:
                    midastype = 'D'
                else:
                    midastype = 'R' 

        # Int type

                       
        if keywordType == types.IntType:
            midastype='I'


        if  midastype == 'C':
             numValues= len( str( midasValues) )

        
        print 'Keyword: ',string     
        print 'no elements: ',numValues
        print 'type: ',midastype
        print 'values: ',midasValues

        #  (02) improved to cut decimals in same lenght as command string
        outputValues = ''   
        for i in midasValues:
            outputValues = outputValues + ',' + i   


        outputValues = outputValues[1:] # remove firts ','
        
        midascommand= "WRITE/KEYW " + string+'/' + midastype+'/1/' + \
                  str( numValues) + ' ' + outputValues 

        #print midascommand
        _do ( midascommand )
        
        
    finally:
        print ''
        #print 'command sent to bkg Midas'




def _validateValue( value ):
    """ Validate Keyword values"""
    valueType = type( value[ 0 ] )# We select the type as the first element
    for i in value:
             if type( i ) != valueType:
                 raise TypeError( 'Elements are not in same type' )
                 return
    else:
        print "valuetype validate=",valueType
        return valueType       


# This is taken from pyraf

def clOscmd(s, **kw):
    """Execute a system-dependent command in the shell, returning status"""

    # handle redirection and save keywords
    redirKW, closeFHList = redirProcess(kw)
    if kw.has_key('_save'): del kw['_save']
    if len(kw):
        raise TypeError('unexpected keyword argument: ' + `kw.keys()`)
    resetList = redirApply(redirKW)
    try:
        # if first character of s is '!' then force to Bourne shell
        if s[:1] == '!':
            shell = "/bin/sh"
            s = s[1:]
        else:
            # otherwise use default shell
            shell=None

        # ignore null commands
        if not s: return 0

       
        # use subshell to execute command so wildcards, etc. are handled
	# version 1.05 catch subprocess errors
        try:
            status = _subproc.subshellRedir(s, shell=shell)
            return status
        except (_subproc.SubprocessError),value:
            print "Warning:"+value

    finally:
        rv = redirReset(resetList, closeFHList)
    return rv



# -----------------------------------------------------
# Utilities to handle I/O redirection keywords
# -----------------------------------------------------

def redirProcess(kw):

    """Process Stdout, Stdin, Stderr keywords used for redirection

    Removes the redirection keywords from kw
    Returns (redirKW, closeFHList) which are a dictionary of
    the filehandles for stdin, stdout, stderr and a list of
    filehandles to close after execution.

    XXX Still need to do graphics redirection keywords
    """

    redirKW = {}
    closeFHList = []
    # Dictionary of redirection keywords
    # Values are (outputFlag, standardName, openArgs)
    # Still need to add graphics redirection keywords
    redirDict = {
                            'Stdin': (0, "stdin", "r"),
                            'Stdout': (1, "stdout", "w"),
                            'StdoutAppend': (1, "stdout", "a"),
                            'Stderr': (1, "stderr", "w"),
                            'StderrAppend': (1, "stderr", "a"),
                            }
    # Magic values that trigger special behavior
    magicValues = { "STDIN": 1, "STDOUT": 1, "STDERR": 1}

    PipeOut = None
    for key in redirDict.keys():
        if kw.has_key(key):
            outputFlag, standardName, openArgs = redirDict[key]
            # if it is a string, open as a file
            # otherwise assume it is a filehandle
            value = kw[key]
            if isinstance(value, _types.StringType):
                if magicValues.has_key(value):
                    if outputFlag and value == "STDOUT":
                        fh = _sys.__stdout__
                    elif outputFlag and value == "STDERR":
                        fh = _sys.__stderr__
                    elif (not outputFlag) and value == "STDIN":
                        fh = _sys.__stdin__
                    else:
                        # MIDAS doesn't raise an exception here (e.g., on
                        # input redirection from "STDOUT"), but it should
                        raise IOError("Illegal value `%s' for %s redirection" %
                                (value, key))
                else:
                    # expand MIDAS variables
                    value = Expand(value)
                    if outputFlag:
                        # output file
                        # check to see if it is dev$null
                        if isNullFile(value):
                            value = '/dev/null'
                        elif "w" in openArgs and \
                          envget("clobber","") != yes and \
                          __os.path.exists(value):
                            # don't overwrite unless clobber is set
                            raise IOError("Output file `%s' already exists" %
                                    value)
                    fh = open(value,openArgs)
                    # close this when we're done
                    closeFHList.append(fh)
            elif isinstance(value, _types.IntType):
                # integer is OK for output keywords -- it indicates
                # that output should be captured and returned as
                # function value
                if not outputFlag:
                    raise os.error("%s redirection must "
                            "be from a file handle or string\n"
                            "Value is `%s'" %
                            (key, value))
                if not value:
                    fh = None
                else:
                    if PipeOut is None:
                        # several outputs can be written to same buffer
                        # (e.g. Stdout=1, Stderr=1 is legal)
                        PipeOut = _StringIO.StringIO()
                        # stick this in the close list too so we know that
                        # output should be returned
                        # wrap it in a tuple to make it easy to recognize
                        closeFHList.append((PipeOut,))
                    fh = PipeOut
            elif type(value) in (_types.ListType, _types.TupleType):
                # list/tuple of strings is OK for input
                if outputFlag:
                    raise os.error("%s redirection must "
                            "be to a file handle or string\n"
                            "Value is type %s" %
                            (key, type(value)))
                try:
                    if value and value[0][-1:] == '\n':
                        s = ''.join(value)
                    else:
                        # ensure there is a newline at the end
                        value.append('')
                        try:
                            s = '\n'.join(value)
                        finally:
                            value.pop()
                    fh = _StringIO.StringIO(s)
                    # close this when we're done
                    closeFHList.append(fh)
                except TypeError:
                    raise PyMidasError("%s redirection must "
                            "be from a sequence of strings\n" % key)
            else:
                # must be a file handle
                if outputFlag:
                    if not hasattr(value, 'write'):
                        raise os.error("%s redirection must "
                                "be to a file handle or string\n"
                                "Value is `%s'" %
                                (key, value))
                elif not hasattr(value, 'read'):
                    raise os.error("%s redirection must "
                            "be from a file handle or string\n"
                            "Value is `%s'" %
                            (key, value))
                fh = value
            if fh is not None: redirKW[standardName] = fh
            del kw[key]
    return redirKW, closeFHList

def redirApply(redirKW):

    """Modify _sys.stdin, stdout, stderr using the redirKW dictionary

    Returns a list of the original filehandles so they can be
    restored (by redirReset)
    """

    sysDict = { 'stdin': 1, 'stdout': 1, 'stderr': 1 }
    resetList = []
    for key, value in redirKW.items():
        if sysDict.has_key(key):
            resetList.append((key, getattr(_sys,key)))
            setattr(_sys,key,value)
    return resetList


def redirReset(resetList, closeFHList):

    """Restore _sys.stdin, stdout, stderr to their original values

    Also closes the filehandles in closeFHList.  If a tuple with a
    StringIO pipe is included in closeFHList, that means the value
    should be returned as the return value of the function.
    Returns an array of lines (without newlines.)
    """
    PipeOut = None
    for fh in closeFHList:
        if isinstance(fh, _types.TupleType):
            PipeOut = fh[0]
        else:
            fh.close()
    for key, value in resetList:
        setattr(_sys,key,value)
    if PipeOut is not None:
        # unfortunately cStringIO.StringIO has no readlines method:
        # PipeOut.seek(0)
        # rv = PipeOut.readlines()
        rv = PipeOut.getvalue().split('\n')
        PipeOut.close()
        # delete trailing null value
        if len(rv)>1 and rv[-1] == '':
            del rv[-1]
        return rv




def Expand(instring, noerror=0):
    """Expand a string

    
    """

    # call _expand1 for each entry in comma-separated list
    wordlist = instring.split(",")
    outlist = []
    for word in wordlist:
        outlist.append(_os.path.expanduser( word ) )
    return ",".join(outlist)





# _backDir is previous working directory

_backDir = None

def pwd(**kw):
    """Print working directory"""
    # handle redirection and save keywords
    redirKW, closeFHList = redirProcess(kw)
    if kw.has_key('_save'): del kw['_save']
    if len(kw):
        raise TypeError('unexpected keyword argument: ' + `kw.keys()`)
    resetList = redirApply(redirKW)
    try:
        print _os.getcwd()
    finally:
        rv = redirReset(resetList, closeFHList)
    return rv

def chdir(directory=None, **kw):
    """Change working directory"""
    # handle redirection and save keywords
    redirKW, closeFHList = redirProcess(kw)
    if kw.has_key('_save'): del kw['_save']
    if len(kw):
        raise TypeError('unexpected keyword argument: ' + `kw.keys()`)
    resetList = redirApply(redirKW)
    try:
        global _backDir
        try:
            _newBack = _os.getcwd()
        except OSError:
            # OSError for getcwd() means current directory does not exist
            _newBack = _backDir
        if directory is None:
            # use startup directory as home if argument is omitted
            directory = _os.environ['PWD']
        # Check for (1) local directory and (2) iraf variable
        # when given an argument like 'dev'.  In IRAF 'cd dev' is
        # the same as 'cd ./dev' if there is a local directory named
        # dev but is equivalent to 'cd dev$' if there is no local
        # directory.
        try:
            edir = Expand(directory)
            _os.chdir(edir)
            _backDir = _newBack
            _do('CHANGE/DIRE  %s' % edir)           
            #clOscmd('pwd')  # show current dir
            return 
        except ( OSError):
            pass
        try:
            edir = Expand(directory + '$')
            _os.chdir(edir)
            _backDir = _newBack
            _do('CHANGE/DIRE  %s' % edir)
            #clOscmd('pwd') # show current dir
            return 
        except ( OSError):
            raise "Cannot change to directory  `%s'" % (directory,)
    finally:
        rv = redirReset(resetList, closeFHList)
    return rv

cd = chdir



def error(errnum=0, errmsg=''):
    """Print error message"""
    raise SyntaxError("ERROR: %s\n" % (errmsg,))
